@@ -11,17 +11,18 @@ import operator
 import os
 import time
 
-
+import sympy
 from deap.base import Fitness
 from deap.tools import HallOfFame, Logbook
 from numpy import random
-
+from sklearn.datasets import load_boston
 from sklearn.metrics import r2_score
 
 from featurebox.symbol.base import CalculatePrecisionSet
 from featurebox.symbol.base import SymbolSet
 from featurebox.symbol.base import SymbolTree
-from featurebox.symbol.functions.dimfunc import dless, Dim
+from featurebox.symbol.calculation.translate import compile_context, general_expr
+from featurebox.symbol.functions.dimfunc import Dim, dless
 from featurebox.symbol.gp import cxOnePoint, varAnd, genGrow, staticLimit, selKbestDim, \
     selTournament, Statis_func, mutUniform, mutShrink, varAndfus, \
     mutDifferentReplacementVerbose, mutNodeReplacementVerbose, selBest, genFull
@@ -35,7 +36,7 @@ class BaseLoop(Toolbox):
 
     def __init__(self, pset, pop=500, gen=20, mutate_prob=0.5, mate_prob=0.8, hall=1, re_hall=None,
                  re_Tree=None, initial_min=None, initial_max=3, max_value=5,
-                 scoring=(r2_score,), score_pen=(1,),filter_warning=True,
+                 scoring=(r2_score,), score_pen=(1,), filter_warning=True, cv=1,
                  add_coef=True, inter_add=True, inner_add=False, vector_add=False,
                  cal_dim=False, dim_type=None, fuzzy=False, n_jobs=1, batch_size=40,
                  random_state=None, stats=None, verbose=True,
@@ -81,6 +82,8 @@ class BaseLoop(Toolbox):
             if multiply score method, the scores must be turn to same dimension in preprocessing
             or weight by score_pen. Because the all the selection are stand on the mean(w_i*score_i)
             Examples: [r2_score] is [1],
+        cv=int,sklearn.model_selection._split._BaseKFold
+            default =1, means not cv
         filter_warning:bool
             filter warning or not
         add_coef:bool
@@ -168,7 +171,7 @@ class BaseLoop(Toolbox):
         self.cpset = CalculatePrecisionSet(pset, scoring=scoring, score_pen=score_pen,
                                            filter_warning=filter_warning, cal_dim=cal_dim,
                                            add_coef=add_coef, inter_add=inter_add, inner_add=inner_add,
-                                           vector_add=vector_add,
+                                           vector_add=vector_add, cv=cv,
                                            n_jobs=n_jobs, batch_size=batch_size, tq=tq,
                                            fuzzy=fuzzy, dim_type=dim_type,
                                            )
@@ -178,11 +181,11 @@ class BaseLoop(Toolbox):
         # def produce
         if initial_min is None:
             initial_min = 2
-        self.register("genGrow", genGrow, pset=self.cpset, min_=initial_min, max_=initial_max,
+        self.register("genGrow", genGrow, pset=self.cpset, min_=initial_min, max_=initial_max+1,
                       personal_map=self.personal_map)
-        self.register("genFull", genFull, pset=self.cpset, min_=initial_min, max_=initial_max,
+        self.register("genFull", genFull, pset=self.cpset, min_=initial_min, max_=initial_max+1,
                       personal_map=self.personal_map)
-        self.register("gen_mu", genGrow, min_=1, max_=3, personal_map=self.personal_map)
+        self.register("gen_mu", genGrow, min_=1, max_=2+1, personal_map=self.personal_map)
         # def selection
 
         self.register("select", selTournament, tournsize=2)
@@ -196,8 +199,8 @@ class BaseLoop(Toolbox):
 
         self.register("mutate", mutUniform, expr=self.gen_mu, pset=self.cpset)
 
-        self.decorate("mate", staticLimit(key=operator.attrgetter("height"), max_value=2 * max_value))
-        self.decorate("mutate", staticLimit(key=operator.attrgetter("height"), max_value=2 * max_value))
+        self.decorate("mate", staticLimit(key=operator.attrgetter("height"), max_value=2 * (max_value+1)))
+        self.decorate("mutate", staticLimit(key=operator.attrgetter("height"), max_value=2 * (max_value+1)))
 
         if stats is None:
             if cal_dim:
@@ -430,7 +433,6 @@ class OnePointMutateLoop(BaseLoop):
 
 
 class DimForceLoop(MutilMutateLoop):
-
     """Force select the individual with target dim for next generation"""
 
     def __init__(self, *args, **kwargs):
@@ -441,65 +443,64 @@ class DimForceLoop(MutilMutateLoop):
         self.register("select", selKbestDim,
                       dim_type=self.cpset.dim_type, fuzzy=self.cpset.fuzzy, force_number=True)
 
+if __name__ == "__main__":
+    # data
+    data = load_boston()
+    x = data["data"]
+    y = data["target"]
+    c = [6, 3, 4]
+    # unit
+    from sympy.physics.units import kg
 
-# if __name__ == "__main__":
-#     # data
-#     data = load_boston()
-#     x = data["data"]
-#     y = data["target"]
-#     c = [6, 3, 4]
-#     # unit
-#     from sympy.physics.units import kg
-#
-#     x_u = [kg] * 13
-#     y_u = kg
-#     c_u = [dless, dless, dless]
-#
-#     x, x_dim = Dim.convert_x(x, x_u, target_units=None, unit_system="SI")
-#     y, y_dim = Dim.convert_xi(y, y_u)
-#     c, c_dim = Dim.convert_x(c, c_u)
-#
-#     z = time.time()
-#
-#     # symbolset
-#     pset0 = SymbolSet()
-#     pset0.add_features(x, y, x_dim=x_dim, y_dim=y_dim, x_group=[[1, 2], [3, 4], [5, 6]])
-#     pset0.add_constants(c, c_dim=c_dim, c_prob=None)
-#     pset0.add_operations(power_categories=(2, 3, 0.5),
-#                          categories=("Add", "Mul", "Sub", "Div", "exp", "Abs"))
-#
-#     # a = time.time()
-#     bl = MutilMutateLoop(pset=pset0, gen=4, pop=10, hall=2, batch_size=40, re_hall=2,
-#                          n_jobs=1, mate_prob=1, max_value=10, initial_max=3,
-#                          mutate_prob=0.8, tq=True, dim_type="coef",
-#                          re_Tree=2, store=False, random_state=1,
-#                          stats={"fitness_dim_max": ["max"], "dim_is_target": ["sum"], "height": ["mean"]},
-#                          add_coef=True, cal_dim=True, inner_add=False, vector_add=True, personal_map=False)
-#     # b = time.time()
-#     bl.run()
-#     population = [bl.PTree(bl.genFull()) for _ in range(30)]
-#     pset = bl.cpset
-#     for i in population:
-#         # i.ppprint(bl.cpset)
-#         # i = "exp(gx0/gx1)"
-#
-#         i = compile_context(i, pset.context, pset.gro_ter_con, simplify=False)
-#         # print(i)
-#         # print(i)
-#         # fun = Coef("V", np.array([1.4,1.3]))
-#         # i = fun(i)
-#         # f = Function("MAdd")
-#         # i = f(i)
-#         try:
-#             # group_str(i,pset)
-#             # i=general_expr(i, pset, simplifying=True)
-#             i = general_expr(i, pset, simplifying=False)
-#             # print(i)
-#             # print(i)
-#             # pprint(i)
-#         except NotImplementedError as e:
-#             print(e)
-#     # c = time.time()
-#     # print(c - b, b - a, a - z)
-#     a, b, c = sympy.Symbol("a"), sympy.Symbol("b"), sympy.Symbol("c")
-#     print(sympy.simplify((a + (b + 1)) * c) == sympy.simplify(a * c + b * c + c))
+    x_u = [kg] * 13
+    y_u = kg
+    c_u = [dless, dless, dless]
+
+    x, x_dim = Dim.convert_x(x, x_u, target_units=None, unit_system="SI")
+    y, y_dim = Dim.convert_xi(y, y_u)
+    c, c_dim = Dim.convert_x(c, c_u)
+
+    z = time.time()
+
+    # symbolset
+    pset0 = SymbolSet()
+    pset0.add_features(x, y, x_dim=x_dim, y_dim=y_dim, x_group=[[1, 2], [3, 4], [5, 6]])
+    pset0.add_constants(c, c_dim=c_dim, c_prob=None)
+    pset0.add_operations(power_categories=(2, 3, 0.5),
+                         categories=("Add", "Mul", "Sub", "Div", "exp", "Abs"))
+
+    # a = time.time()
+    bl = MutilMutateLoop(pset=pset0, gen=4, pop=10, hall=2, batch_size=40, re_hall=2,
+                         n_jobs=1, mate_prob=1, max_value=10, initial_max=3,
+                         mutate_prob=0.8, tq=True, dim_type="coef",
+                         re_Tree=2, store=False, random_state=1,
+                         stats={"fitness_dim_max": ["max"], "dim_is_target": ["sum"], "height": ["mean"]},
+                         add_coef=True, cal_dim=True, inner_add=False, vector_add=True, personal_map=False)
+    # b = time.time()
+    bl.run()
+    population = [bl.PTree(bl.genFull()) for _ in range(30)]
+    pset = bl.cpset
+    for i in population:
+        # i.ppprint(bl.cpset)
+        # i = "exp(gx0/gx1)"
+
+        i = compile_context(i, pset.context, pset.gro_ter_con, simplify=False)
+        # print(i)
+        # print(i)
+        # fun = Coef("V", np.array([1.4,1.3]))
+        # i = fun(i)
+        # f = Function("MAdd")
+        # i = f(i)
+        try:
+            # group_str(i,pset)
+            # i=general_expr(i, pset, simplifying=True)
+            i = general_expr(i, pset, simplifying=False)
+            # print(i)
+            # print(i)
+            # pprint(i)
+        except NotImplementedError as e:
+            print(e)
+    # c = time.time()
+    # print(c - b, b - a, a - z)
+    a, b, c = sympy.Symbol("a"), sympy.Symbol("b"), sympy.Symbol("c")
+    print(sympy.simplify((a + (b + 1)) * c) == sympy.simplify(a * c + b * c + c))
